/****************************************************************************
** Meta object code from reading C++ file 'kdeplatformfiledialogbase_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../plasma-integration-6.6.5/qt5/src/platformtheme/kdeplatformfiledialogbase_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kdeplatformfiledialogbase_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDEPlatformFileDialogBase_t {
    QByteArrayData data[14];
    char stringdata0[153];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDEPlatformFileDialogBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDEPlatformFileDialogBase_t qt_meta_stringdata_KDEPlatformFileDialogBase = {
    {
QT_MOC_LITERAL(0, 0, 25), // "KDEPlatformFileDialogBase"
QT_MOC_LITERAL(1, 26, 6), // "closed"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "fileSelected"
QT_MOC_LITERAL(4, 47, 4), // "file"
QT_MOC_LITERAL(5, 52, 13), // "filesSelected"
QT_MOC_LITERAL(6, 66, 11), // "QList<QUrl>"
QT_MOC_LITERAL(7, 78, 5), // "files"
QT_MOC_LITERAL(8, 84, 14), // "currentChanged"
QT_MOC_LITERAL(9, 99, 4), // "path"
QT_MOC_LITERAL(10, 104, 16), // "directoryEntered"
QT_MOC_LITERAL(11, 121, 9), // "directory"
QT_MOC_LITERAL(12, 131, 14), // "filterSelected"
QT_MOC_LITERAL(13, 146, 6) // "filter"

    },
    "KDEPlatformFileDialogBase\0closed\0\0"
    "fileSelected\0file\0filesSelected\0"
    "QList<QUrl>\0files\0currentChanged\0path\0"
    "directoryEntered\0directory\0filterSelected\0"
    "filter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDEPlatformFileDialogBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    1,   45,    2, 0x06 /* Public */,
       5,    1,   48,    2, 0x06 /* Public */,
       8,    1,   51,    2, 0x06 /* Public */,
      10,    1,   54,    2, 0x06 /* Public */,
      12,    1,   57,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl,   11,
    QMetaType::Void, QMetaType::QString,   13,

       0        // eod
};

void KDEPlatformFileDialogBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KDEPlatformFileDialogBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closed(); break;
        case 1: _t->fileSelected((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: _t->filesSelected((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 3: _t->currentChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->directoryEntered((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->filterSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (KDEPlatformFileDialogBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::closed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (KDEPlatformFileDialogBase::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::fileSelected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (KDEPlatformFileDialogBase::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::filesSelected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (KDEPlatformFileDialogBase::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::currentChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (KDEPlatformFileDialogBase::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::directoryEntered)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (KDEPlatformFileDialogBase::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KDEPlatformFileDialogBase::filterSelected)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KDEPlatformFileDialogBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_KDEPlatformFileDialogBase.data,
    qt_meta_data_KDEPlatformFileDialogBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDEPlatformFileDialogBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPlatformFileDialogBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDEPlatformFileDialogBase.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KDEPlatformFileDialogBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KDEPlatformFileDialogBase::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KDEPlatformFileDialogBase::fileSelected(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDEPlatformFileDialogBase::filesSelected(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDEPlatformFileDialogBase::currentChanged(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDEPlatformFileDialogBase::directoryEntered(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void KDEPlatformFileDialogBase::filterSelected(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
