/* Generated by wayland-scanner 1.25.0 */

/*
 * SPDX-FileCopyrightText: 2017 David Edmundson
 *
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface org_kde_kwin_appmenu_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *appmenu_types[] = {
	NULL,
	NULL,
	&org_kde_kwin_appmenu_interface,
	&wl_surface_interface,
};

static const struct wl_message org_kde_kwin_appmenu_manager_requests[] = {
	{ "create", "no", appmenu_types + 2 },
	{ "release", "2", appmenu_types + 0 },
};

WL_EXPORT const struct wl_interface org_kde_kwin_appmenu_manager_interface = {
	"org_kde_kwin_appmenu_manager", 2,
	2, org_kde_kwin_appmenu_manager_requests,
	0, NULL,
};

static const struct wl_message org_kde_kwin_appmenu_requests[] = {
	{ "set_address", "ss", appmenu_types + 0 },
	{ "release", "", appmenu_types + 0 },
};

WL_EXPORT const struct wl_interface org_kde_kwin_appmenu_interface = {
	"org_kde_kwin_appmenu", 2,
	2, org_kde_kwin_appmenu_requests,
	0, NULL,
};

