// This file was generated by qtwaylandscanner
// source file is /usr/share/plasma-wayland-protocols/appmenu.xml

#ifndef QT_WAYLAND_APPMENU
#define QT_WAYLAND_APPMENU

#include "wayland-appmenu-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {
    class  org_kde_kwin_appmenu_manager
    {
    public:
        org_kde_kwin_appmenu_manager(struct ::wl_registry *registry, int id, int version);
        org_kde_kwin_appmenu_manager(struct ::org_kde_kwin_appmenu_manager *object);
        org_kde_kwin_appmenu_manager();

        virtual ~org_kde_kwin_appmenu_manager();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::org_kde_kwin_appmenu_manager *object);

        struct ::org_kde_kwin_appmenu_manager *object() { return m_org_kde_kwin_appmenu_manager; }
        const struct ::org_kde_kwin_appmenu_manager *object() const { return m_org_kde_kwin_appmenu_manager; }
        static org_kde_kwin_appmenu_manager *fromObject(struct ::org_kde_kwin_appmenu_manager *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

        struct ::org_kde_kwin_appmenu *create(struct ::wl_surface *surface);
        void release();

    private:
        struct ::org_kde_kwin_appmenu_manager *m_org_kde_kwin_appmenu_manager;
    };

    class  org_kde_kwin_appmenu
    {
    public:
        org_kde_kwin_appmenu(struct ::wl_registry *registry, int id, int version);
        org_kde_kwin_appmenu(struct ::org_kde_kwin_appmenu *object);
        org_kde_kwin_appmenu();

        virtual ~org_kde_kwin_appmenu();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::org_kde_kwin_appmenu *object);

        struct ::org_kde_kwin_appmenu *object() { return m_org_kde_kwin_appmenu; }
        const struct ::org_kde_kwin_appmenu *object() const { return m_org_kde_kwin_appmenu; }
        static org_kde_kwin_appmenu *fromObject(struct ::org_kde_kwin_appmenu *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

        void set_address(const QString &service_name, const QString &object_path);
        void release();

    private:
        struct ::org_kde_kwin_appmenu *m_org_kde_kwin_appmenu;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
