/****************************************************************************
** Meta object code from reading C++ file 'khintssettings.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../plasma-integration-6.6.5/qt5/src/platformtheme/khintssettings.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'khintssettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KHintsSettings_t {
    QByteArrayData data[13];
    char stringdata0[152];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KHintsSettings_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KHintsSettings_t qt_meta_stringdata_KHintsSettings = {
    {
QT_MOC_LITERAL(0, 0, 14), // "KHintsSettings"
QT_MOC_LITERAL(1, 15, 19), // "delayedDBusConnects"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 15), // "setupIconLoader"
QT_MOC_LITERAL(4, 52, 19), // "toolbarStyleChanged"
QT_MOC_LITERAL(5, 72, 16), // "slotNotifyChange"
QT_MOC_LITERAL(6, 89, 4), // "type"
QT_MOC_LITERAL(7, 94, 3), // "arg"
QT_MOC_LITERAL(8, 98, 24), // "slotPortalSettingChanged"
QT_MOC_LITERAL(9, 123, 5), // "group"
QT_MOC_LITERAL(10, 129, 3), // "key"
QT_MOC_LITERAL(11, 133, 12), // "QDBusVariant"
QT_MOC_LITERAL(12, 146, 5) // "value"

    },
    "KHintsSettings\0delayedDBusConnects\0\0"
    "setupIconLoader\0toolbarStyleChanged\0"
    "slotNotifyChange\0type\0arg\0"
    "slotPortalSettingChanged\0group\0key\0"
    "QDBusVariant\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KHintsSettings[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x08 /* Private */,
       3,    0,   40,    2, 0x08 /* Private */,
       4,    0,   41,    2, 0x08 /* Private */,
       5,    2,   42,    2, 0x08 /* Private */,
       8,    3,   47,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    6,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 11,    9,   10,   12,

       0        // eod
};

void KHintsSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KHintsSettings *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->delayedDBusConnects(); break;
        case 1: _t->setupIconLoader(); break;
        case 2: _t->toolbarStyleChanged(); break;
        case 3: _t->slotNotifyChange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->slotPortalSettingChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KHintsSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KHintsSettings.data,
    qt_meta_data_KHintsSettings,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KHintsSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHintsSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KHintsSettings.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KHintsSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
