/* Generated by wayland-scanner 1.25.0 */

/*
 * SPDX-FileCopyrightText: 2017 David Edmundson
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface org_kde_kwin_server_decoration_palette_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *server_decoration_palette_types[] = {
	NULL,
	&org_kde_kwin_server_decoration_palette_interface,
	&wl_surface_interface,
};

static const struct wl_message org_kde_kwin_server_decoration_palette_manager_requests[] = {
	{ "create", "no", server_decoration_palette_types + 1 },
};

WL_PRIVATE const struct wl_interface org_kde_kwin_server_decoration_palette_manager_interface = {
	"org_kde_kwin_server_decoration_palette_manager", 1,
	1, org_kde_kwin_server_decoration_palette_manager_requests,
	0, NULL,
};

static const struct wl_message org_kde_kwin_server_decoration_palette_requests[] = {
	{ "set_palette", "s", server_decoration_palette_types + 0 },
	{ "release", "", server_decoration_palette_types + 0 },
};

WL_PRIVATE const struct wl_interface org_kde_kwin_server_decoration_palette_interface = {
	"org_kde_kwin_server_decoration_palette", 1,
	2, org_kde_kwin_server_decoration_palette_requests,
	0, NULL,
};

