/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p statusnotifierwatcher_interface /build/plasma-integration/src/plasma-integration-6.6.5/qt5/src/platformtheme/org.kde.StatusNotifierWatcher.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "statusnotifierwatcher_interface.h"

/*
 * Implementation of interface class OrgKdeStatusNotifierWatcherInterface
 */

OrgKdeStatusNotifierWatcherInterface::OrgKdeStatusNotifierWatcherInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeStatusNotifierWatcherInterface::~OrgKdeStatusNotifierWatcherInterface()
{
}


#include "statusnotifierwatcher_interface.moc"
